@@ -14,6 +14,13 @@ R_json_parse(SEXP str)
     return(R_MakeExternalPtr(node, Rf_install("JSONNODE"), R_NilValue));
 }
 
+void
+jsonErrorHandler(const json_char *foo)
+{
+    PROBLEM  "json error handler: %s", foo
+      ERROR;
+}
+
 SEXP
 R_fromJSON(SEXP r_str, SEXP simplify, SEXP nullValue, SEXP simplifyWithNames, SEXP encoding,
             SEXP r_stringFun, SEXP r_str_type)
@@ -36,6 +43,11 @@ R_fromJSON(SEXP r_str, SEXP simplify, SEXP nullValue, SEXP simplifyWithNames, SE
     } else 
 	r_stringFun = NULL;
 
+#if 0
+    register_debug_callback(jsonErrorHandler);
+#else
+#pragma message("activate the debug_callback")
+#endif
     node = json_parse(str);
     ans = processJSONNode(node, json_type(node), INTEGER(simplify)[0], nullValue, LOGICAL(simplifyWithNames)[0],
  			  INTEGER(encoding)[0], r_stringFun, str_fun_type);
@@ -46,6 +58,17 @@ R_fromJSON(SEXP r_str, SEXP simplify, SEXP nullValue, SEXP simplifyWithNames, SE
     return(ans);
 }
 
+int 
+getElType(int len, int numNumbers, int numStrings, int numLogicals, int numNulls)
+{
+    if(numStrings > 0) 
+	return(STRSXP);
+    if(numNumbers > 0) 
+	return(REALSXP);
+    return(LGLSXP);
+}
+
+
 SEXP 
 processJSONNode(JSONNODE *n, int parentType, int simplify, SEXP nullValue, int simplifyWithNames, cetype_t charEncoding,
                  SEXP r_stringCall, StringFunctionType str_fun_type)
@@ -221,14 +244,17 @@ processJSONNode(JSONNODE *n, int parentType, int simplify, SEXP nullValue, int s
 	    ((TYPEOF(nullValue) == LGLSXP && LOGICAL(nullValue)[0] == NA_INTEGER) && 
 	     ((numNumbers + numNulls) == len || (numStrings + numNulls) == len || (numLogicals + numNulls) == len));
         homogeneous = allSame ||  ( (numNumbers + numStrings + numLogicals + numNulls) == len);
+
         if(simplify == NONE) {
 	} else if(allSame && 
 		  ((numNumbers == len && (simplify & STRICT_NUMERIC)) ||
   		      ((numLogicals == len) && (simplify & STRICT_LOGICAL)) ||
 		   ( (numStrings == len) && (simplify & STRICT_CHARACTER)))) {
+	       elType = getElType(len, numNumbers, numStrings, numLogicals, numNulls);
    	       ans = makeVector(ans, len, elType, nullValue);
 	} else if((simplify == ALL && homogeneous) || (simplify == STRICT && allSame)) {
-   	       ans = makeVector(ans, len, elType, nullValue);
+	    elType = getElType(len, numNumbers, numStrings, numLogicals, numNulls);
+	    ans = makeVector(ans, len, elType, nullValue);
 	}
     }
       
